with Ada.Text_IO, System;
use  Ada.Text_IO;

procedure audiocd is

   -- Sample program for playing audio CD's


   -- DEVICES
   --
   -- This section deals with device files, in particular,
   -- the cdrom device

   DevCDROM : constant string := "/dev/cdrom";
   -- path to the CDROM device, usually /dev/cdrom

   type ioctlID is new integer;
   type aFileID is new integer;
   -- Define these as separate types for error checking.
   -- A ioctlID is never the same as a FileID.

   type byte is new integer range 0..255;
   for byte'size use 8;

   CDROMPLAYTRKIND : constant ioctlID := 16#5304#;
   CDROMSTOP   : constant ioctlID := 16#5307#;
   CDROMSTART  : constant ioctlID := 16#5308#;
   -- various CDROM ioctl functions as mentioned in the
   -- CDROM documentation in /usr/doc/kernel...

   type aDummyParam is new integer;
   -- define this as a separate type to make sure nothing
   -- important is used as a third parameter to ioctl_noparam

   -- a version of ioctl for functions that don't
   -- use a third parameter

   procedure ioctl_noparam( result : out integer;
        fid : aFileID;
        id : ioctlID;
        ignored : in out aDummyParam );
   pragma import( C, ioctl_noparam, "ioctl" );
   pragma import_valued_procedure( ioctl_noparam );

   type cdrom_ti is record
        start_track, start_index : byte;
        end_track, end_index : byte;
   end record;
   --pragma pack( cdrom_ti );
   -- from /usr/src/linux/include/linux/cdrom.h

   -- PLAYTRKIND ioctl function uses cdrom_ti record

   procedure ioctl_playtrkind( result : out integer;
        fid : aFileID;
        id : ioctlID;
        info : in out cdrom_ti );
   pragma import( C, ioctl_playtrkind, "ioctl" );
   pragma import_valued_procedure( ioctl_playtrkind );


   -- KERNEL CALLS
   --
   -- Calls to the Linux kernel (besides ioctl).

   procedure open( id : out aFileID;
        path : string;
        flags : integer );
   pragma import( C, open, "open" );
   pragma import_valued_procedure( open );
   -- open is a kernel call to open a file

   procedure close( result : out integer; id : aFileID );
   pragma import( C, close, "close" );
   pragma import_valued_procedure( close );
   -- close is a kernel call to close a file


   -- C LIBRARY CALLS
   --
   -- Calls to the standard Linux C libraries

   procedure perror( prefixstr : string );
   pragma import( C, perror, "perror" );
   -- perror is a standard C library call to print
   -- the last error message from a kernel call or the
   -- standard C libraries on the screen

   cd : aFileID;
   playinfo : cdrom_ti;
   dummy : ADummyParam;
   ioctl_result : integer;
   close_result : integer;

   ch : character;

begin
  Put_Line( "This program plays an audio CD in your CDROM drive" );
  New_Line;

  -- open the /dev/cdrom file so we can control the CDROM drive
  -- using ioctl
 
  Put_Line( "Openning " & DevCDROM & "..." ); 
  Open( cd, DevCDROM & ASCII.NUL, 0 );
  if cd < 0 then
     perror( "Error openning CDROM drive" );
  end if;

  -- start the CDROM drive

  Put_Line( "Spinning up cdrom..." ); 
  ioctl_noparam( ioctl_result, cd, CDROMSTART, dummy );
  if ioctl_result < 0 then
     perror( "Error spinning up the CDROM drive" );
  end if;

  -- display menu

  New_Line;
  Put_Line( "1 = Play, 2 = Quit" );
  New_Line;

  -- Main loop.  Repeat until 2 is selected.

  loop
     Put( "Select a function (1-2): " );
     Get( ch );
     case ch is
       when '1' => playinfo.start_track := 1; -- first track
                   playinfo.start_index := 0; -- no effect
                   playinfo.end_track := 9;   -- final track (inclusive)
                   playinfo.end_index := 0;   -- no effect
                   ioctl_playtrkind( ioctl_result, cd,
                      CDROMPLAYTRKIND, playinfo );
       when '2' => ioctl_noparam( ioctl_result, cd, CDROMSTOP, dummy );
                   exit;
       when others => Put_Line( "Pardon?" );
     end case;
     if ioctl_result < 0 then
        perror( "Error controlling CDROM drive" );
     end if;

  end loop;

  -- Close the CDROM device

  Close( close_result, cd );
  if close_result < 0 then
     perror( "Error controlling CDROM drive" );
  end if;
end audiocd;

