------------------------------------------------------------------------------
-- Bindings to the SDL (Simple DirectMedia Layer) Library                   --
--                                                                          --
-- Part of BUSH                                                             --
------------------------------------------------------------------------------
--                                                                          --
--              Copyright (C) 2001-2004 Ken O. Burtch & FSF                 --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  This is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- This is maintained at http://www.pegasoft.ca                             --
--                                                                          --
------------------------------------------------------------------------------
-- CVS: $Header: /home/cvsroot/ada/bush/bush_linux.ads,v 1.1.1.1 2003/10/16 03:07:48 ken Exp $

with Interfaces.C, System.Address_To_Access_Conversions;
use  Interfaces.C;

package bush_os.sdl is

------------------------------------------------------------------------------
-- Standard SDL Types ( from /usr/include/SDL/SDL_types.h)
------------------------------------------------------------------------------

type C_string is new char_array(0..1024);
-- length arbitrarily chosen.  fixed bounds required for C interfacing

subtype SDL_bool is boolean;
-- SDL bool is the same as an Ada boolean

type SDL_success is new int;
SDL_OK     : constant SDL_success := 0;
SDL_Failed : constant SDL_success := -1;

type Uint8 is new unsigned_char;   for Uint8'size  use 8;
type Sint8 is new signed_char;     for Sint8'size  use 8;
type Uint16 is new unsigned_short; for Uint16'size use 16;
type Sint16 is new short;          for Sint16'size use 16;
type Uint32 is new unsigned;       for Uint32'size use 32;
type Sint32 is new int;            for Sint32'size use 32;
-- constrain the proper number of bits

type private_ptr is new SYSTEM.ADDRESS;
-- generic pointer not to be dereferenced

------------------------------------------------------------------------------
-- Housekeeping (from /usr/include/SDL/SDL.h)
------------------------------------------------------------------------------

type SDL_Init_Flags is new Uint32;
SDL_INIT_TIMER       : constant SDL_Init_Flags := 16#00000001#;
SDL_INIT_AUDIO       : constant SDL_Init_Flags := 16#00000010#;
SDL_INIT_VIDEO       : constant SDL_Init_Flags := 16#00000020#;
SDL_INIT_CDROM       : constant SDL_Init_Flags := 16#00000100#;
SDL_INIT_JOYSTICK    : constant SDL_Init_Flags := 16#00000200#;
SDL_INIT_NOPARACHUTE : constant SDL_Init_Flags := 16#00100000#;
SDL_INIT_EVENTTHREAD : constant SDL_Init_Flags := 16#01000000#;
SDL_INIT_EVERYTHING  : constant SDL_Init_Flags := 16#0000FFFF#;

procedure SDL_Init( flags : SDL_Init_Flags );
pragma import( C, SDL_Init, "SDL_Init" );

function SDL_WasInit( flags : SDL_Init_Flags) return SDL_Init_Flags;
pragma import( C, SDL_WasInit, "SDL_WasInit" );

procedure SDL_Quit;
pragma import( C, SDL_Quit, "SDL_Quit" );

type SDL_version is record
     major : Uint8;
     minor : Uint8;
     patch : Uint8;
end record;
-- SDL_version is a C preprocessor macro so we can't call it from Ada anyway


------------------------------------------------------------------------------
-- Video Subprograms (from /usr/include/SDL/SDL_video.h)
------------------------------------------------------------------------------

type SDL_Generic_Pixel is new Uint32;
type SDL_RGB_Component is new Uint8;
type SDL_RGB_Mask is new Uint32;

type alpha_transparency is new Uint8;
SDL_ALPHA_OPAQUE      : constant alpha_transparency := 255;
SDL_ALPHA_TRANSPARENT : constant alpha_transparency := 0;

type SDL_Rect is record
     x, y : Sint16;
     w, h : Uint16;
end record;
pragma pack( SDL_Rect );
 
package SDL_Rect_Conv is new
     system.address_to_access_conversions( SDL_Rect );
subtype SDL_Rect_Ptr is SDL_Rect_Conv.object_pointer;
--package SDL_Rect_Ptr_Conv is new
--     system.address_to_access_conversions( SDL_Rect_Ptr );
--subtype SDL_Rect_Handle is SDL_Rect_Ptr_Conv.object_pointer;

type SDL_Color is record
     r, g, b, unused : SDL_RGB_Component;
end record;
pragma pack( SDL_Color );
subtype SDL_Colour is SDL_Color;

package SDL_Color_Conv is new
     system.address_to_access_conversions( SDL_Color );
subtype SDL_Color_Ptr is SDL_Color_Conv.object_pointer;

type SDL_Palette is record
     ncolors : int;
     colors  : SDL_Color_Ptr; -- actually, an array of colours
end record;
pragma pack( SDL_Palette );

package SDL_Palette_Conv is new
     system.address_to_access_conversions( SDL_Palette );
subtype SDL_Palette_Ptr is SDL_Palette_Conv.object_pointer;

type SDL_PixelFormat is record
        palette : SDL_Palette_Ptr;
        BitsPerPixel : Uint8;
        BytesPerPixel : Uint8;
        Rloss : Uint8;
        Gloss : Uint8;
        Bloss : Uint8;
        Aloss : Uint8;
        Rshift: Uint8;
        Gshift: Uint8;
        Bshift: Uint8;
        Ashift: Uint8;
        Rmask : SDL_RGB_Mask;
        Gmask : SDL_RGB_Mask;
        Bmask : SDL_RGB_Mask;
        Amask : SDL_RGB_Mask;
        colorkey : SDL_Generic_Pixel;
        alpha : alpha_transparency;
end record;
pragma pack( SDL_PixelFormat );

package SDL_PixelFormat_Conv is new
     system.address_to_access_conversions( SDL_PixelFormat );
subtype SDL_PixelFormat_Ptr is SDL_PixelFormat_Conv.object_pointer;

type SDL_Surface_Flag is new Uint32;
SDL_SWSURFACE   : constant SDL_Surface_Flag := 16#00000000#;
SDL_HWSURFACE   : constant SDL_Surface_Flag := 16#00000001#;
SDL_ASYNCBLIT   : constant SDL_Surface_Flag := 16#00000004#;
SDL_ANYFORMAT   : constant SDL_Surface_Flag := 16#10000000#;
SDL_HWPALETTE   : constant SDL_Surface_Flag := 16#20000000#;
SDL_DOUBLEBUF   : constant SDL_Surface_Flag := 16#40000000#;
SDL_FULLSCREEN  : constant SDL_Surface_Flag := 16#80000000#;
SDL_OPENGL      : constant SDL_Surface_Flag := 16#00000002#;
SDL_OPENGLBLIT  : constant SDL_Surface_Flag := 16#0000000A#;
SDL_RESIZABLE   : constant SDL_Surface_Flag := 16#00000010#;
SDL_NOFRAME     : constant SDL_Surface_Flag := 16#00000020#;
SDL_HWACCEL     : constant SDL_Surface_Flag := 16#00000100#;
SDL_SRCCOLORKEY : constant SDL_Surface_Flag := 16#00001000#;
SDL_RLEACCELOK  : constant SDL_Surface_Flag := 16#00002000#;
SDL_RLEACCEL    : constant SDL_Surface_Flag := 16#00004000#;
SDL_SRCALPHA    : constant SDL_Surface_Flag := 16#00010000#;
SDL_PREALLOC    : constant SDL_Surface_Flag := 16#01000000#;

type SDL_Pixels is array( natural range <> ) of Uint8;
package SDL_Pixels_Conv is new
     system.address_to_access_conversions( SDL_Pixels );
subtype SDL_Pixels_Ptr is SDL_Pixels_Conv.object_pointer;
package SDL_Pixels_Ptr_Conv is new
     system.address_to_access_conversions( SDL_Pixels_Ptr );
subtype SDL_Pixels_Handle is SDL_Pixels_Ptr_Conv.object_pointer;
-- we don't know that the pixels are 8-bits but use Uint8--it's the smallest.
-- use unchecked_conversion, etc to access the pixels

type SDL_Surface is record
        flags     : SDL_Surface_Flag;
        format    : System.Address;
        w, h      : int;
        pitch     : Uint16;
        pixels    : System.Address;
        offset    : int;
        hwdata    : System.Address;
        clip_rect : SDL_Rect;
        unused1   : Uint32;
        locked    : Uint32;
        map       : Private_Ptr;
        format_version : unsigned;
        refcount  : int;
end record;
pragma pack( SDL_Surface );

package SDL_Surface_Conv is new
     system.address_to_access_conversions( SDL_Surface );
subtype SDL_Surface_Ptr is SDL_Surface_Conv.object_pointer;

type SDL_VideoInfo is record
        hw_available  : boolean;
        wm_available  : boolean;
        UnusedBits1_1 : boolean;
        UnusedBits1_2 : boolean;
        UnusedBits1_3 : boolean;
        UnusedBits1_4 : boolean;
        UnusedBits1_5 : boolean;
        UnusedBits1_6 : boolean;
        UnusedBits2   : boolean;
        blit_hw       : boolean;
        blit_hw_CC    : boolean;
        blit_hw_A     : boolean;
        blit_sw       : boolean;
        blit_sw_CC    : boolean;
        blit_sw_A     : boolean;
        blit_fill     : boolean;
        UnusedBits3   : Uint16;
        video_mem     : Uint32;
        vfmt          : System.Address;
end record;
pragma pack( SDL_VideoInfo );
-- this may only pack booleans to bytes, not bits.  investigate.

package SDL_VideoInfo_Conv is new
     system.address_to_access_conversions( SDL_VideoInfo );
subtype SDL_VideoInfo_Ptr is SDL_VideoInfo_Conv.object_pointer;

type SDL_Overlay_Type is new Uint32;
SDL_YV12_OVERLAY : constant SDL_Overlay_type := 16#32315659#;
SDL_IYUV_OVERLAY : constant SDL_Overlay_type := 16#56555949#;
SDL_YUY2_OVERLAY : constant SDL_Overlay_type := 16#32595559#;
SDL_UYVY_OVERLAY : constant SDL_Overlay_type := 16#59565955#;
SDL_YVYU_OVERLAY : constant SDL_Overlay_type := 16#55595659#;

type SDL_Pitches is array( natural range <> ) of Uint16;
package SDL_Pitches_Conv is new
     system.address_to_access_conversions( SDL_Pitches );
subtype SDL_Pitches_Ptr is SDL_Pitches_Conv.object_pointer;
-- may not be Uint8 but we'll use this as the base.

type SDL_Overlay is record
        format : SDL_Overlay_Type;
        w, h   : int;
        planes : int;
        pitches: System.Address;
        pixels : System.Address;
        hwfuncs: Private_Ptr;
        hwdata : Private_Ptr;
        hw_overlay : Uint32; -- only bit 1 used
end record;
pragma pack( SDL_Overlay );

package SDL_Overlay_Conv is new
     system.address_to_access_conversions( SDL_Overlay );
subtype SDL_Overlay_Ptr is SDL_Overlay_Conv.object_pointer;

type SDL_GLattr is (
    SDL_GL_RED_SIZE,
    SDL_GL_GREEN_SIZE,
    SDL_GL_BLUE_SIZE,
    SDL_GL_ALPHA_SIZE,
    SDL_GL_BUFFER_SIZE,
    SDL_GL_DOUBLEBUFFER,
    SDL_GL_DEPTH_SIZE,
    SDL_GL_STENCIL_SIZE,
    SDL_GL_ACCUM_RED_SIZE,
    SDL_GL_ACCUM_GREEN_SIZE,
    SDL_GL_ACCUM_BLUE_SIZE,
    SDL_GL_ACCUM_ALPHA_SIZE,
    SDL_GL_STEREO );
pragma convention( C, SDL_GLattr );

type SDL_Palette_Type is new Uint32;
SDL_LOGPAL  : constant SDL_Palette_Type := 16#01#;
SDL_PHYSPAL : constant SDL_Palette_Type := 16#02#;

procedure SDL_VideoDriverName( result : out private_ptr; nameBuff : in out C_string; maxLen : int );
pragma import( C, SDL_VideoDriverName, "SDL_VideoDriverName" );
pragma import_valued_procedure( SDL_VideoDriverName );

function SDL_GetVideoSurface return System.Address;
pragma import( C, SDL_GetVideoSurface, "SDL_GetVideoSurface" );

function SDL_GetVideoInfo return System.Address;
pragma import( C, SDL_GetVideoInfo, "SDL_GetVideoInfo" );

function SDL_VideoModeOK( width, height, bpp : int; flags : System.Address) return int;
pragma import( C, SDL_VideoModeOK, "SDL_VideoModeOK" );

procedure SDL_ListModes( result : out System.Address; format : in out SDL_PixelFormat; flags : SDL_Surface_Flag );
pragma import( C, SDL_ListModes, "SDL_ListModes" );
pragma import_valued_procedure( SDL_ListModes );
-- result maybe should be a handle, not a pointer

function SDL_SetVideoMode( width, height, bpp : int; flags : SDL_Surface_Flag) return System.Address;
pragma import( C, SDL_SetVideoMode, "SDL_SetVideoMode" );

procedure SDL_UpdateRects( screen : System.Address;  numrects : int; rects : System.Address );
pragma import( C, SDL_UpdateRects, "SDL_UpdateRects" );
-- really, a pointer to a rect array

procedure SDL_UpdateRect( screen : System.Address; x, y : Sint32; w, h : Uint32);
pragma import( C, SDL_UpdateRect, "SDL_UpdateRect" );

function SDL_Flip( screen : System.Address ) return SDL_success;
pragma import( C, SDL_Flip, "SDL_Flip" );

function SDL_SetGamma( red, green, blue : float ) return SDL_success;
pragma import( C, SDL_SetGamma, "SDL_SetGamma" );

type SDL_Gamma_Ramp is new Uint16;
package SDL_Gamma_Conv is new
     system.address_to_access_conversions( SDL_Gamma_Ramp );
subtype SDL_Gamma_Ramp_Ptr is SDL_Gamma_Conv.object_pointer;
SDL_Gamma_Ramp_Null : constant SYSTEM.Address := SDL_Gamma_Conv.To_Address( null );

procedure SDL_SetGammaRamp( result : out SDL_success; red, green, blue : System.Address );
pragma import( C, SDL_SetGammaRamp, "SDL_SetGammaRamp" );
pragma import_valued_procedure( SDL_SetGammaRamp );

procedure SDL_GetGammaRamp( result : out SDL_success; red, green, blue : System.Address );
pragma import( C, SDL_GetGammaRamp, "SDL_GetGammaRamp" );
pragma import_valued_procedure( SDL_GetGammaRamp );

function SDL_SetColors( surface : System.Address; colors : System.Address; firstcolor, ncolors : int ) return SDL_bool;
pragma import( C, SDL_SetColors, "SDL_SetColors" );

function SDL_SetPalette( surface : System.Address; flags : SDL_Palette_Type; colors : System.Address; firstcolor, ncolors : int) return SDL_bool;
pragma import( C, SDL_SetPalette, "SDL_SetPalette" );

procedure SDL_MapRGB( result : out SDL_Generic_Pixel; format : System.Address; r, g, b : SDL_RGB_Component );
pragma import( C, SDL_MapRGB, "SDL_MapRGB" );
pragma import_valued_procedure( SDL_MapRGB );

procedure SDL_MapRGBA( result : out SDL_Generic_Pixel; format : System.Address; r, g, b, a : SDL_RGB_Component );
pragma import( C, SDL_MapRGBA, "SDL_MapRGBA" );

procedure SDL_GetRGB( pixel : SDL_Generic_Pixel; fmt : System.Address; r, g, b : in out SDL_RGB_Component );
pragma import( C, SDL_GetRGB, "SDL_GetRGB" );

procedure SDL_GetRGBA( pixel : SDL_Generic_Pixel; fmt : System.Address; r, g, b, a : in out SDL_RGB_Component );
pragma import( C, SDL_GetRGBA, "SDL_GetRGBA" );

function SDL_CreateRGBSurface( flags : SDL_Surface_Flag; width, height, depth : int; Rmask, Gmask, Bmask, Amask : SDL_RGB_Mask ) return System.Address;
pragma import( C, SDL_CreateRGBSurface, "SDL_CreateRGBSurface" );

function SDL_CreateRGBSurfaceFrom( pixels : System.Address; width, height, depth, pitch : int; Rmask, Gmask, Bmask, Amask : SDL_RGB_Mask ) return System.Address;
pragma import( C, SDL_CreateRGBSurfaceFrom, "SDL_CreateRGBSurfaceFrom" );

procedure SDL_FreeSurface( surface : System.Address );
pragma import( C, SDL_FreeSurface, "SDL_FreeSurface" );

function SDL_LockSurface( surface : System.Address ) return SDL_success;
pragma import( C, SDL_LockSurface, "SDL_LockSurface" );

procedure SDL_UnlockSurface( surface : System.Address );
pragma import( C, SDL_UnlockSurface, "SDL_UnlockSurface" );

function SDL_LoadBMP_RW( src : System.Address; freesrc : SDL_Bool ) return System.Address;
pragma import( C, SDL_LoadBMP_RW, "SDL_LoadBMP_RW" );

function SDL_SaveBMP_RW( surface : System.Address; dst : System.Address; freedst : SDL_Bool ) return SDL_Success;
pragma import( C, SDL_SaveBMP_RW, "SDL_SaveBMP_RW" );

function SDL_SetColorKey( surface : System.Address; flag : SDL_Surface_Flag; key : SDL_Generic_Pixel ) return SDL_Success;
pragma import( C, SDL_SetColorKey, "SDL_SetColorKey" );

function SDL_SetAlpha( surface : System.Address; flag : SDL_Surface_Flag; alpha : SDL_RGB_Component ) return SDL_Success;
pragma import( C, SDL_SetAlpha, "Set_SetAlpha" );
                                                                                
function SDL_SetClipRect( surface : System.Address; rect : System.Address) return SDL_Bool;
pragma import( C, SDL_SetClipRect, "Set_SetClipRect" );

procedure SDL_GetClipRect( surface : System.Address; rect : in out SDL_Rect);
pragma import( C, SDL_GetClipRect, "Set_GetClipRect" );

function SDL_ConvertSurface( src : System.Address; fmt : System.Address; flags : SDL_Surface_Flag ) return System.Address;
pragma import( C, SDL_ConvertSurface, "SDL_ConvertSurface" );

function SDL_UpperBlit( src : System.Address; srcrect : System.Address; dst : System.Address; dstrect : System.Address ) return SDL_Success;
pragma import( C, SDL_UpperBlit, "SDL_UpperBlit" );

function SDL_LowerBlit( src : System.Address; srcrect : System.Address; dst : System.Address; dstrect : System.Address ) return SDL_Success;
pragma import( C, SDL_LowerBlit, "SDL_LowerBlit" );

function SDL_FillRect( dst : System.Address; dstrect : System.Address; color : SDL_Generic_Pixel ) return SDL_Success;
pragma import( C, SDL_FillRect, "SDL_FillRect" );

function SDL_DisplayFormat( surface : System.Address ) return System.Address;
pragma import( C, SDL_DisplayFormat, "SDL_DisplayFormat" );

function SDL_DisplayFormatAlpha( surface : System.Address ) return System.Address;
pragma import( C, SDL_DisplayFormatAlpha, "SDL_DisplayFormatAlpha" );

-- YUV Overlays

function SDL_CreateYUVOverlay( width, height : int; format : SDL_Overlay_Type; display : System.Address ) return System.Address;
pragma import( C, SDL_CreateYUVOverlay, "SDL_CreateYUVOverlay" );

function SDL_LockYUVOverlay( Overlay : System.Address) return SDL_Success;
pragma import( C, SDL_LockYUVOverlay, "SDL_LockYUVOverlay" );

procedure SDL_UnlockYUVOverlay( Overall : System.Address);
pragma import( C, SDL_UnlockYUVOverlay, "SDL_UnlockYUVOverlay" );

function SDL_DisplayYUVOverlay( Overlay : System.Address; DstRect : System.Address ) return boolean;
pragma import( C, SDL_DisplayYUVOverlay, "SDL_DisplayYUVOverlay" );

procedure SDL_FreeYUVOverlay( Overlay : System.Address );
pragma import( C, SDL_FreeYUVOverlay, "SDL_FreeYUVOverlay" );

-- Open/GL Interface

function SDL_GL_LoadLibrary( path : C_string ) return boolean;
pragma import( C, SDL_GL_LoadLibrary, "SDL_GL_LoadLibrary" );

function SDL_GL_GetProcAddress( proc : C_string) return Private_Ptr;
pragma import( C, SDL_GL_GetProcAddress, "SDL_GL_GetProcAddress" );

function SDL_GL_SetAttribute( attr : SDL_GLattr; value : int) return SDL_Success;
pragma import( C, SDL_GL_SetAttribute, "SDL_GL_SetAttribute" );

procedure SDL_GL_GetAttribute( result : out SDL_Success; attr : SDL_GLattr; value : in out int);
pragma import( C, SDL_GL_GetAttribute, "SDL_GL_GetAttribute" );
pragma import_valued_procedure( SDL_GL_GetAttribute );

procedure SDL_GL_SwapBuffers;
pragma import( C, SDL_GL_SwapBuffers, "SDL_GL_SwapBuffers" );

procedure SDL_GL_UpdateRects( numrects : int; rects : System.Address );
pragma import( C, SDL_GL_UpdateRects, "SDL_GL_UpdateRects" );

procedure SDL_GL_Lock;
pragma import( C, SDL_GL_Lock, "SDL_GL_Lock" );

procedure SDL_GL_Unlock;
pragma import( C,  SDL_GL_Unlock, "SDL_GL_Unlock" );

-- Window Manager Functions

procedure SDL_WM_SetCaption( title : C_String; icon : Private_Ptr);
pragma import( C, SDL_WM_SetCaption, "SDL_WM_SetCaption" );

procedure SDL_WM_GetCaption( title : in out C_String; icon : in out Private_Ptr );
pragma import( C, SDL_WM_GetCaption, "SDL_WM_GetCaption" );

procedure SDL_WM_SetIcon(icon : System.Address; mask : SDL_RGB_Component );
pragma import( C, SDL_WM_SetIcon );

function SDL_WM_IconifyWindow return SDL_bool;
pragma import( C, SDL_WM_IconifyWindow, "SDL_WM_IconifyWindow" );

function SDL_WM_ToggleFullScreen( surface : System.Address ) return SDL_Bool;
pragma import( C, SDL_WM_ToggleFullScreen, "SDL_WM_ToggleFullScreen" );

type SDL_GrabMode is new int;
SDL_GRAB_QUERY      : constant SDL_GrabMode := -1;
SDL_GRAB_OFF        : constant SDL_GrabMode := 0;
SDL_GRAB_ON         : constant SDL_GrabMode := 1;
SDL_GRAB_FULLSCREEN : constant SDL_GrabMode := 2;

function SDL_WM_GrabInput(mode : SDL_GrabMode) return SDL_GrabMode;
pragma import( C, SDL_WM_GrabInput, "SDL_WM_GrabInput" );

end bush_os.sdl;

