/* C code for functions not covered in bush_os.ads          */
/* Part of BUSH                                             */
/*                                                          */
/* I didn't want to resort to C, but I need to use stat()   */
/* and manually declaring the structure in Ada is extremely */
/* unportable between UNIXes -- KB                          */

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <time.h>
#include <errno.h>

/* C ERRNO                                                  */
/*                                                          */
/* Patch for GCC 3.x / GNAT 5.x which won't import errno.   */

int C_errno( void ) {
    return errno;
}

/* C RESET ERRNO                                            */
/*                                                          */
/* Patch for GCC 3.x / GNAT 5.x which won't import errno.   */

void C_reset_errno( void ) {
     errno = 0;
}

/* IS EXECUTABLE FILE                                       */
/*                                                          */
/* True if file exists, is regular, and is executable       */

int C_is_executable_file( char * path ) {
  struct stat info;
  int result = 0;
  if ( stat( path, &info ) == 0 ) {
     if ( info.st_mode & S_IFREG ) {
        if ( getuid() == info.st_uid ) {
           result = (info.st_mode & S_IXUSR ) > 0;
        } else if ( getgid() == info.st_gid ) {
           result = (info.st_mode & S_IXGRP ) > 0;
        } else {
           result = (info.st_mode & S_IXOTH ) > 0;
        }
     }
  }
  return result;
}

/* IS READABLE FILE                                         */
/*                                                          */
/* True if file exists, is regular, and is executable       */

int C_is_readable_file( char * path ) {
  struct stat info;
  int result = 0;
  if ( stat( path, &info ) == 0 ) {
     if ( info.st_mode & S_IFREG ) {
        if ( getuid() == info.st_uid ) {
           result = (info.st_mode & S_IRUSR ) > 0;
        } else if ( getgid() == info.st_gid ) {
           result = (info.st_mode & S_IRGRP ) > 0;
        } else {
           result = (info.st_mode & S_IROTH ) > 0;
        }
     }
  }
  return result;
}

/* IS WAITING FILE                                          */
/*                                                          */
/* True if file exists, is regular, readable and not empty  */

int C_is_waiting_file( char * path ) {
  struct stat info;
  int result = 0;
  if ( stat( path, &info ) == 0 ) {
     if ( info.st_mode & S_IFREG ) {
        if ( info.st_size > 0 ) {
           if ( getuid() == info.st_uid ) {
              result = (info.st_mode & S_IRUSR ) > 0;
           } else if ( getgid() == info.st_gid ) {
              result = (info.st_mode & S_IRGRP ) > 0;
           } else {
              result = (info.st_mode & S_IROTH ) > 0;
           }
        }
     }
  }
  return result;
}

/* FILE LENGTH                                              */
/*                                                          */
/* Return file length as reported by stat().                */

int C_file_length( char * path ) {
  struct stat info;
  int result = -1;
  if ( stat( path, &info ) == 0 )
     result = info.st_size;
  return result;
}

/* FILE MODIFY TIME                                         */
/*                                                          */
/* Return file modify time as reported by stats().          */

void C_file_modify_time( char * path, int *year, int *month, int *day, int *sec ) {
  struct stat info;
  time_t result = -1;
  struct tm *timestruct;

  *year = -1;
  if ( stat( path, &info ) == 0 ) {
     result = info.st_mtime;
     timestruct = localtime( &info.st_mtime );
     *year = timestruct->tm_year+1900;
     *month = timestruct->tm_mon+1;
     *day = timestruct->tm_mday;
     *sec = timestruct->tm_sec+
            timestruct->tm_min*60+
            timestruct->tm_hour*60*60;
  }
}

/* FILE CHANGE TIME                                         */
/*                                                          */
/* Return file change time as reported by stats().          */

void C_file_change_time( char * path, int *year, int *month, int *day, int *sec ) {
  struct stat info;
  time_t result = -1;
  struct tm *timestruct;

  *year = -1;
  if ( stat( path, &info ) == 0 ) {
     result = info.st_mtime;
     timestruct = localtime( &info.st_ctime );
     *year = timestruct->tm_year+1900;
     *month = timestruct->tm_mon+1;
     *day = timestruct->tm_mday;
     *sec = timestruct->tm_sec+
            timestruct->tm_min*60+
            timestruct->tm_hour*60*60;
  }
}

/* FILE ACCESS TIME                                         */
/*                                                          */
/* Return file access time as reported by stats().          */

void C_file_access_time( char * path, int *year, int *month, int *day, int *sec ) {
  struct stat info;
  time_t result = -1;
  struct tm *timestruct;

  *year = -1;
  if ( stat( path, &info ) == 0 ) {
     result = info.st_mtime;
     timestruct = localtime( &info.st_atime );
     *year = timestruct->tm_year+1900;
     *month = timestruct->tm_mon+1;
     *day = timestruct->tm_mday;
     *sec = timestruct->tm_sec+
            timestruct->tm_min*60+
            timestruct->tm_hour*60*60;
  }
}

/* DAY OF WEEK                                              */
/*                                                          */
/* Return the day of week                                   */

void C_day_of_week( int * wday, int year, int month, int day ) {
  time_t result = -1;
  struct tm timestruct;
  struct tm *timestruct2;

  *wday = -1;
  timestruct.tm_year = year-1900;
  timestruct.tm_mon  = month-1;
  timestruct.tm_mday = day;
  timestruct.tm_sec  = 0;
  timestruct.tm_min  = 0;
  timestruct.tm_hour = 0;
  result = mktime( &timestruct );
  timestruct2 = localtime( &result );
  *wday = timestruct2->tm_wday+1;

}

// SIGNAL HANDLING

int *sigchld_flag = 0;         // Address of Ada boolean variable
int *sigint_flag = 0;          // Address of Ada boolean variable
int *sigwinch_flag = 0;          // Address of Ada boolean variable

void sigint_handler( int sig ) {
  *sigint_flag = 1;
}

int C_install_sigint_handler( int *flag_address ) {
  static unsigned int handler_installed = 0;
  sigset_t signalmask;
  struct sigaction old_sigint;
  struct sigaction sa;
  int res = 0;

  sigint_flag = flag_address;                    // where to flag occurence
  sigfillset( &signalmask );                     // block all during handling
  sa.sa_handler = sigint_handler;                // what to do on signal
  sa.sa_mask    = signalmask;                    // mask all signals
  sa.sa_flags   = 0;                             // trap zombie children
  res = sigaction( SIGINT, &sa, &old_sigint );   // setup signal trap
  if ( res == 0 )                                // OK?
     handler_installed = 1;                      // mark as installed
  return handler_installed;
}

void sigchld_handler( int sig ) {
  *sigchld_flag = 1;
}

int C_install_sigchld_handler( int *flag_address ) {
  static unsigned int handler_installed = 0;
  sigset_t signalmask;
  struct sigaction old_sigchld;
  struct sigaction sa;
  int res = 0;

  sigchld_flag = flag_address;                   // where to flag occurence
  sigfillset( &signalmask );                     // block all during handling
  sa.sa_handler = sigchld_handler;               // what to do on signal
  sa.sa_mask    = signalmask;                    // mask all signals
  sa.sa_flags   = 0;                             // trap zombie children
  res = sigaction( SIGCHLD, &sa, &old_sigchld ); // setup signal trap
  if ( res == 0 )                                // OK?
     handler_installed = 1;                      // mark as installed
  return handler_installed;
}

void sigwinch_handler( int sig ) {
  *sigwinch_flag = 1;
}

int C_install_sigwinch_handler( int *flag_address ) {
  static unsigned int handler_installed = 0;
  sigset_t signalmask;
  struct sigaction old_sigwinch;
  struct sigaction sa;
  int res = 0;

  sigwinch_flag = flag_address;                  // where to flag occurence
  sigfillset( &signalmask );                     // block all during handling
  sa.sa_handler = sigwinch_handler;              // what to do on signal
  sa.sa_mask    = signalmask;                    // mask all signals
  sa.sa_flags   = 0;                             // trap zombie children
  res = sigaction( SIGWINCH, &sa, &old_sigwinch ); // setup signal trap
  if ( res == 0 )                                // OK?
     handler_installed = 1;                      // mark as installed
  return handler_installed;
}


/* end of c_os.c */
