------------------------------------------------------------------------------
-- BUSH SDL (Simple Direct Media Layer) Package Parser                      --
--                                                                          --
-- Part of BUSH                                                             --
------------------------------------------------------------------------------
--                                                                          --
--              Copyright (C) 2001-2004 Ken O. Burtch & FSF                 --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  This is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- This is maintained at http://www.pegasoft.ca                             --
--                                                                          --
------------------------------------------------------------------------------
-- CVS: $Header$

with text_io;use text_io;
with world,
    scanner,
    string_util,
    parser_aux,
    parser,
    bush_os.sdl;
use world, 
    scanner,
    string_util,
    parser_aux,
    parser,
    bush_os,
    bush_os.sdl;

package body parser_sdl is

procedure ParseSDLInit is
  -- Syntax: Lock_File( dir, file [,wait [,retry] ) or
  --         Lock_File( file, [,wait [,retry] ] )
  expr_val   : unbounded_string;
  expr_type  : identifier;
  dir_val    : unbounded_string;
  file_val   : unbounded_string;
  wait_val   : unbounded_string := to_unbounded_string( "1.0" );
  retry_val  : unbounded_string := to_unbounded_string( natural'last'img );
begin
  expect( sdl_init_t );
  expect( symbol_t, "(" );
  ParseExpression( expr_val, expr_type );
  if baseTypesOk( expr_type, string_t ) then
     if token = symbol_t and identifiers( token ).value = ")" then
        file_val := expr_val;
     else
        expect( symbol_t, "," );
        ParseExpression( file_val, expr_type );
        -- first variation: file [,wait [,retry] ]
        if getUniType( expr_type ) = uni_string_t then
           dir_val := expr_val;
           if token = symbol_t and identifiers( token ).value = "," then
              expect( symbol_t, "," );
              ParseExpression( wait_val, expr_type );
              if baseTypesOk( expr_type, duration_t ) then
                 if token = symbol_t and identifiers( token ).value = "," then
                    expect( symbol_t, "," );
                    ParseExpression( retry_val, expr_type );
                    if baseTypesOk( expr_type, duration_t ) then
                       null;
                    end if;
                 end if;
              end if;
           end if;
     elsif baseTypesOk( expr_type, duration_t ) then
        -- second variation: [,wait [,retry] ]
        file_val := expr_val;
        if token = symbol_t and identifiers( token ).value = "," then
           expect( symbol_t, "," );
           ParseExpression( wait_val, expr_type );
           if baseTypesOk( expr_type, duration_t ) then
              if token = symbol_t and identifiers( token ).value = "," then
                 ParseExpression( retry_val, expr_type );
                 if baseTypesOk( expr_type, duration_t ) then
                    null;
                 end if;
              end if;
           end if;
        end if;
     end if;
     end if;
  end if;
  expect( symbol_t, ")" );
  if isExecutingCommand then
     begin
       null;
     exception when others =>
       err( "exception raised" );
     end;
  end if;
end ParseSDLInit;

procedure ParseSDLQuit is
  -- Syntax: unlock_file( file ) or
  --         unlock_file( dir, file )
  dir_val    : unbounded_string := null_unbounded_string;
  file_val   : unbounded_string;
  expr_type  : identifier;
begin
  expect( sdl_quit_t );
  expect( symbol_t, "(" );
  ParseExpression( file_val, expr_type );
  if baseTypesOk( expr_type, string_t ) then
     if token = symbol_t and identifiers( token ).value = "," then
        expect( symbol_t, "," );
        dir_val := file_val;
        ParseExpression( file_val, expr_type );
        if baseTypesOk( expr_type, string_t ) then
           null;
        end if;
     end if;
  end if;
  expect( symbol_t, ")" );
  if isExecutingCommand then
     null;
  end if;
end ParseSDLQuit;

procedure StartupSdl is
begin
  declareIdent( sdl_init_flag_t, "sdl.init_flag", integer_t, typeClass );
  declareStandardConstant( sdl_init_timer_t, "sdl.init_timer",
    sdl_init_flag_t, "1" );
  declareStandardConstant( sdl_init_audio_t, "sdl.init_audio",
    sdl_init_flag_t, "16" );
  declareStandardConstant( sdl_init_video_t, "sdl.init_video",
    sdl_init_flag_t, "32" );
  declareStandardConstant( sdl_init_cdrom_t, "sdl.init_cdrom",
    sdl_init_flag_t, "256" );
  declareStandardConstant( sdl_init_joystick_t, "sdl.init_joystick",
    sdl_init_flag_t, "512" );
  declareStandardConstant( sdl_no_parachute_t, "sdl.no_parachute",
    sdl_init_flag_t, "1048576" );
  declareStandardConstant( sdl_eventthread_t, "sdl.eventthread",
    sdl_init_flag_t, "16777216" );
  declareStandardConstant( sdl_eventthing_t, "sdl.everything",
    sdl_init_flag_t, "65535" );

  declareProcedure( sdl_init_t, "sdl.init" );
  declareProcedure( sdl_quit_t, "sdl.quit" );

end StartupSdl;

procedure ShutdownSdl is
begin
  null;
end ShutdownSdl;

end parser_sdl;
